/*
Copyright 2019 Benji Dial

Permission to use, copy, modify, and/or distribute this
software for any purpose with or without fee is hereby
granted, provided that the above copyright notice and this
permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
OF THIS SOFTWARE.
*/

#include "vga.h"
#include "files.h"
#include "mem.h"
#include "proc.h"
#include "misc.h"

void _start(void) {
  *(uint16_t *)0x0000200e = 0x0000;

  *(uint32_t *)0x0000050c = (uint32_t)allocate_pages(16 * sizeof(struct file_handle_info), 2);
  for (struct file_handle_info *i = FILE_HANDLES, *l = FILE_HANDLES + 65536; i < l; ++i)
    i->first_sector = 0;

  *(uint32_t *)0x00000510 = (uint32_t)allocate_pages(16 * sizeof(struct proc_info), 2);
  for (struct proc_info *i = PROCS, *l = PROCS + 65536; i < l; ++i)
    i->memory_start = 0;

  uint16_t pages_free = 0, pages_total = 0;
  for (uint16_t *i = MMAP; i < MMAP + 0x1000; ++i)
    switch (*i) {
    case 1:
      continue;
    case 0:
      ++pages_free;
    default:
      ++pages_total;
    }
  put_16(pages_free);
  put_sz(" of ");
  put_16(pages_total);
  put_sz(" pages free.\n");

  put_sz("Welcome to Portland version 0.0.8!\n");
  while (1)
    put_sz("|\b/\b-\b\\\b");
}
